@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2015, Juniper Networks, Inc.
+ * Copyright (c) 2014-2018, Juniper Networks, Inc.
  * All rights reserved.
  * This SOFTWARE is licensed under the LICENSE provided in the
  * ../Copyright file. By downloading, installing, copying, or otherwise
@@ -23,6 +23,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <stdarg.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <errno.h>
 
@@ -39,7 +40,10 @@
  * coward's path, we'll turn it on inside a #if that allows
  * others to turn it off where needed.  Not ideal, but functional.
  */
-#if !defined(NO_PRINTFLIKE) && !defined(__linux__)
+#if !defined(NO_PRINTFLIKE)
+#if defined(__linux) && !defined(__printflike)
+#define __printflike(_x, _y) __attribute__((__format__ (__printf__, _x, _y)))
+#endif
 #define PRINTFLIKE(_x, _y) __printflike(_x, _y)
 #else
 #define PRINTFLIKE(_x, _y)
@@ -97,6 +101,9 @@ typedef unsigned long long xo_xof_flags_t;
 #define XOF_RETAIN_ALL	XOF_BIT(30) /** Force use of XOEF_RETAIN */
 #define XOF_RETAIN_NONE	XOF_BIT(31) /** Prevent use of XOEF_RETAIN */
 
+#define XOF_COLOR_MAP	XOF_BIT(32) /** Color map has been initialized */
+#define XOF_CONTINUATION XOF_BIT(33) /** Continuation of previous line */
+
 typedef unsigned xo_emit_flags_t; /* Flags to xo_emit() and friends */
 #define XOEF_RETAIN	(1<<0)	  /* Retain parsed formatting information */
 
@@ -115,7 +122,18 @@ typedef struct xo_info_s {
 struct xo_handle_s;		/* Opaque structure forward */
 typedef struct xo_handle_s xo_handle_t; /* Handle for XO output */
 
-typedef int (*xo_write_func_t)(void *, const char *);
+/*
+ * Early versions of the API used "int" instead of "size_t" for buffer
+ * sizes.  We want to fix this but allow for backwards compatibility
+ * where needed.
+ */
+#ifdef XO_USE_INT_RETURN_CODES
+typedef int xo_ssize_t;		/* Buffer size */
+#else /* XO_USE_INT_RETURN_CODES */
+typedef ssize_t xo_ssize_t;	/* Buffer size */
+#endif /* XO_USE_INT_RETURN_CODES */
+
+typedef xo_ssize_t (*xo_write_func_t)(void *, const char *);
 typedef void (*xo_close_func_t)(void *);
 typedef int (*xo_flush_func_t)(void *);
 typedef void *(*xo_realloc_func_t)(void *, size_t);
@@ -126,7 +144,7 @@ typedef void (*xo_free_func_t)(void *);
  * of the xo handle.  The caller should return the number of bytes _needed_
  * to fit the data, even if this exceeds 'len'.
  */
-typedef int (*xo_formatter_t)(xo_handle_t *, char *, int,
+typedef xo_ssize_t (*xo_formatter_t)(xo_handle_t *, char *, xo_ssize_t,
 				const char *, va_list);
 typedef void (*xo_checkpointer_t)(xo_handle_t *, va_list, int);
 
@@ -182,56 +200,56 @@ xo_set_formatter (xo_handle_t *xop, xo_formatter_t func, xo_checkpointer_t);
 void
 xo_set_depth (xo_handle_t *xop, int depth);
 
-int
+xo_ssize_t
 xo_emit_hv (xo_handle_t *xop, const char *fmt, va_list vap);
 
-int
+xo_ssize_t
 xo_emit_h (xo_handle_t *xop, const char *fmt, ...);
 
-int
+xo_ssize_t
 xo_emit (const char *fmt, ...);
 
-int
+xo_ssize_t
 xo_emit_hvf (xo_handle_t *xop, xo_emit_flags_t flags,
 	     const char *fmt, va_list vap);
 
-int
+xo_ssize_t
 xo_emit_hf (xo_handle_t *xop, xo_emit_flags_t flags, const char *fmt, ...);
 
-int
+xo_ssize_t
 xo_emit_f (xo_emit_flags_t flags, const char *fmt, ...);
 
 PRINTFLIKE(2, 0)
-static inline int
+static inline xo_ssize_t
 xo_emit_hvp (xo_handle_t *xop, const char *fmt, va_list vap)
 {
     return xo_emit_hv(xop, fmt, vap);
 }
 
 PRINTFLIKE(2, 3)
-static inline int
+static inline xo_ssize_t
 xo_emit_hp (xo_handle_t *xop, const char *fmt, ...)
 {
     va_list vap;
     va_start(vap, fmt);
-    int rc = xo_emit_hv(xop, fmt, vap);
+    xo_ssize_t rc = xo_emit_hv(xop, fmt, vap);
     va_end(vap);
     return rc;
 }
 
 PRINTFLIKE(1, 2)
-static inline int
+static inline xo_ssize_t
 xo_emit_p (const char *fmt, ...)
 {
     va_list vap;
     va_start(vap, fmt);
-    int rc = xo_emit_hv(NULL, fmt, vap);
+    xo_ssize_t rc = xo_emit_hv(NULL, fmt, vap);
     va_end(vap);
     return rc;
 }
 
 PRINTFLIKE(3, 0)
-static inline int
+static inline xo_ssize_t
 xo_emit_hvfp (xo_handle_t *xop, xo_emit_flags_t flags,
 	      const char *fmt, va_list vap)
 {
@@ -239,118 +257,127 @@ xo_emit_hvfp (xo_handle_t *xop, xo_emit_flags_t flags,
 }
 
 PRINTFLIKE(3, 4)
-static inline int
+static inline xo_ssize_t
 xo_emit_hfp (xo_handle_t *xop, xo_emit_flags_t flags, const char *fmt, ...)
 {
     va_list vap;
     va_start(vap, fmt);
-    int rc = xo_emit_hvf(xop, flags, fmt, vap);
+    xo_ssize_t rc = xo_emit_hvf(xop, flags, fmt, vap);
     va_end(vap);
     return rc;
 }
 
 PRINTFLIKE(2, 3)
-static inline int
+static inline xo_ssize_t
 xo_emit_fp (xo_emit_flags_t flags, const char *fmt, ...)
 {
     va_list vap;
     va_start(vap, fmt);
-    int rc = xo_emit_hvf(NULL, flags, fmt, vap);
+    xo_ssize_t rc = xo_emit_hvf(NULL, flags, fmt, vap);
     va_end(vap);
     return rc;
 }
 
-int
+xo_ssize_t
+xo_open_container_hf (xo_handle_t *xop, xo_xof_flags_t flags, const char *name);
+
+xo_ssize_t
 xo_open_container_h (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_open_container (const char *name);
 
-int
+xo_ssize_t
 xo_open_container_hd (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_open_container_d (const char *name);
 
-int
+xo_ssize_t
 xo_close_container_h (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_close_container (const char *name);
 
-int
+xo_ssize_t
 xo_close_container_hd (xo_handle_t *xop);
 
-int
+xo_ssize_t
 xo_close_container_d (void);
 
-int
+xo_ssize_t
+xo_open_list_hf (xo_handle_t *xop, xo_xof_flags_t flags, const char *name);
+
+xo_ssize_t
 xo_open_list_h (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_open_list (const char *name);
 
-int
+xo_ssize_t
 xo_open_list_hd (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_open_list_d (const char *name);
 
-int
+xo_ssize_t
 xo_close_list_h (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_close_list (const char *name);
 
-int
+xo_ssize_t
 xo_close_list_hd (xo_handle_t *xop);
 
-int
+xo_ssize_t
 xo_close_list_d (void);
 
-int
+xo_ssize_t
+xo_open_instance_hf (xo_handle_t *xop, xo_xof_flags_t flags, const char *name);
+
+xo_ssize_t
 xo_open_instance_h (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_open_instance (const char *name);
 
-int
+xo_ssize_t
 xo_open_instance_hd (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_open_instance_d (const char *name);
 
-int
+xo_ssize_t
 xo_close_instance_h (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_close_instance (const char *name);
 
-int
+xo_ssize_t
 xo_close_instance_hd (xo_handle_t *xop);
 
-int
+xo_ssize_t
 xo_close_instance_d (void);
 
-int
+xo_ssize_t
 xo_open_marker_h (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_open_marker (const char *name);
 
-int
+xo_ssize_t
 xo_close_marker_h (xo_handle_t *xop, const char *name);
 
-int
+xo_ssize_t
 xo_close_marker (const char *name);
 
-int
+xo_ssize_t
 xo_attr_h (xo_handle_t *xop, const char *name, const char *fmt, ...);
 
-int
+xo_ssize_t
 xo_attr_hv (xo_handle_t *xop, const char *name, const char *fmt, va_list vap);
 
-int
+xo_ssize_t
 xo_attr (const char *name, const char *fmt, ...);
 
 void
@@ -362,16 +389,16 @@ xo_error_h (xo_handle_t *xop, const char *fmt, ...);
 void
 xo_error (const char *fmt, ...);
 
-int
+xo_ssize_t
 xo_flush_h (xo_handle_t *xop);
 
-int
+xo_ssize_t
 xo_flush (void);
 
-int
+xo_ssize_t
 xo_finish_h (xo_handle_t *xop);
 
-int
+xo_ssize_t
 xo_finish (void);
 
 void
@@ -644,16 +671,16 @@ char *
 xo_simplify_format (xo_handle_t *xop, const char *fmt, int with_numbers,
 		    xo_simplify_field_func_t field_cb);
 
-int
+xo_ssize_t
 xo_emit_field_hv (xo_handle_t *xop, const char *rolmod, const char *contents,
 		  const char *fmt, const char *efmt,
 		  va_list vap);
 
-int
+xo_ssize_t
 xo_emit_field_h (xo_handle_t *xop, const char *rolmod, const char *contents,
 		 const char *fmt, const char *efmt, ...);
 
-int
+xo_ssize_t
 xo_emit_field (const char *rolmod, const char *contents,
 	       const char *fmt, const char *efmt, ...);
 
