@@ -15,6 +15,7 @@
 
 #include "xo_config.h"
 #include "xo.h"
+#include "xo_encoder.h"
 
 int
 main (int argc, char **argv)
@@ -29,25 +30,25 @@ main (int argc, char **argv)
 	return 1;
 
     for (argc = 1; argv[argc]; argc++) {
-	if (strcmp(argv[argc], "xml") == 0)
+	if (xo_streq(argv[argc], "xml"))
 	    xo_set_style(NULL, XO_STYLE_XML);
-	else if (strcmp(argv[argc], "json") == 0)
+	else if (xo_streq(argv[argc], "json"))
 	    xo_set_style(NULL, XO_STYLE_JSON);
-	else if (strcmp(argv[argc], "text") == 0)
+	else if (xo_streq(argv[argc], "text"))
 	    xo_set_style(NULL, XO_STYLE_TEXT);
-	else if (strcmp(argv[argc], "html") == 0)
+	else if (xo_streq(argv[argc], "html"))
 	    xo_set_style(NULL, XO_STYLE_HTML);
-	else if (strcmp(argv[argc], "no-color") == 0)
+	else if (xo_streq(argv[argc], "no-color"))
 	    opt_color = 0;
-	else if (strcmp(argv[argc], "pretty") == 0)
+	else if (xo_streq(argv[argc], "pretty"))
 	    xo_set_flags(NULL, XOF_PRETTY);
-	else if (strcmp(argv[argc], "xpath") == 0)
+	else if (xo_streq(argv[argc], "xpath"))
 	    xo_set_flags(NULL, XOF_XPATH);
-	else if (strcmp(argv[argc], "info") == 0)
+	else if (xo_streq(argv[argc], "info"))
 	    xo_set_flags(NULL, XOF_INFO);
-	else if (strcmp(argv[argc], "no-retain") == 0)
+	else if (xo_streq(argv[argc], "no-retain"))
 	    flags &= ~XOEF_RETAIN;
-	else if (strcmp(argv[argc], "big") == 0) {
+	else if (xo_streq(argv[argc], "big")) {
 	    if (argv[argc + 1])
 		count = atoi(argv[++argc]);
 	}
